using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// Draws a ring from the center of the screen that expands over time
// Ring-drawing requires a LineRenderer component
// Ring-edge-detection requires an EdgeCollider2D
[RequireComponent(typeof(LineRenderer))]
[RequireComponent(typeof(EdgeCollider2D))]
public class Ring : Projectile
{
    [SerializeField] private float lineWidth = .2f; // How thick the line should be
    [SerializeField] private int numberOfPoints = 360; // Number of points in the circle. More points = smoother circle. Less points = chunkier circle.
    
    private LineRenderer lineRenderer; // Needed to draw the circle
    private Vector3[] rendererPositions; // Needed by the line renderer to draw the circle
    private Vector2[] colliderPositions; // Needed to set the edge collider positions, as EdgeCollider2D.points is an array of Vector2 objects.
    private float radius = 0.1f; // How large the circle is
    [SerializeField] private float maxRadius = 4.5f; // How large the circle can be
    
    private Color endColor = new Color(0f, 0f, 0f, .6f); // The color of the ring as it fades out
    
    private void Awake()
    {
        lineRenderer = this.GetComponent<LineRenderer>();
        ConfigureLineRendererSettings();
        DrawCircle(radius);
        StartCoroutine(UpdateCircle());
    }
    
    // -----------------
    // COLLISION FUNCTION
    // -----------------
    
    public virtual void OnTriggerEnter2D (Collider2D other)
    {
        if (other.gameObject.tag == "Enemy")
        {
            CheckEnemyColor(other.gameObject);
        }
    }
    
    // -----------------
    // DRAWING FUNCTIONS
    // -----------------    
    
    // Updates the radius of the circle over time
    private IEnumerator UpdateCircle()
    {
        while (radius < maxRadius)
        {
            radius += movementSpeed * Time.deltaTime;
            DrawCircle(radius);
            this.GetComponent<EdgeCollider2D>().points = colliderPositions;
            if (isColor1)
            {
                lineRenderer.startColor = Color.Lerp(color1, endColor, radius / maxRadius);
                lineRenderer.endColor = Color.Lerp(color1, endColor, radius / maxRadius);
            }
            else
            {
                lineRenderer.startColor = Color.Lerp(color2, endColor, radius / maxRadius);
                lineRenderer.endColor = Color.Lerp(color2, endColor, radius / maxRadius);
            }
            yield return null;
        }
        Destroy(this.gameObject);
    }
    
    // Determines how thick the circle-drawing line is
    private void SetLineWidth(LineRenderer lineRenderer)
    {
        lineRenderer.startWidth = lineWidth;
        lineRenderer.endWidth = lineWidth;
    }
    
    // Draws a circle on the screen
    private void DrawCircle (float radius)
    {
        float angle = 2 * Mathf.PI / numberOfPoints;
        for (int i = 0; i < numberOfPoints; i++)
        {
            float x = this.transform.position.x + Mathf.Cos(angle * i) * radius;
            float y = this.transform.position.y + Mathf.Sin(angle * i) * radius;
            rendererPositions[i] = new Vector3(x, y, 0f);
            colliderPositions[i] = new Vector2(x, y);
        }
        lineRenderer.SetPositions(rendererPositions);
    }
    
    // -----------------
    // MISC FUNCTIONS
    // -----------------  
    
    // Changes the color of the circle
    public override void SetProjectileColor(bool whichColor)
    {
        isColor1 = whichColor;
        if (isColor1)
        {
            lineRenderer.startColor = color1;
            lineRenderer.endColor = color1;
        }
        else
        {
            lineRenderer.startColor = color2;
            lineRenderer.endColor = color2;
        }
    }
    
    // Sets up the line renderer
    public void ConfigureLineRendererSettings()
    {
        SetLineWidth(lineRenderer);
        lineRenderer.loop = true;
        lineRenderer.positionCount = numberOfPoints;
        rendererPositions = new Vector3[numberOfPoints];
        colliderPositions = new Vector2[numberOfPoints];
    }
}
